#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "titles.h"

static void roll(int dice[5], int mask[5]) {
  int i;
  for (i=0; i<5; i++)
    if (mask[i])
      dice[i] = rand() % 6 + 1;
}

static void get_reroll(int dice[5], int mask[5]) {
  char buf[100];
  int i;

  while (1) {
    printf("Which do you want to reroll? "); fflush(stdout);
    gets(buf);
    mask[0] = mask[1] = mask[2] = mask[3] = mask[4] = 0;
    for (i=0; buf[i] != '\0'; i++) {
      if (buf[i] >= '0' && buf[i] <= '9') {
        int j;
        for (j=0; j<5; j++)
          if (buf[i] - '0' == dice[j] && !mask[j]) {
            mask[j] = 1;
            buf[i] = -1;
          }
      }
      else
        buf[i] = -1;
      if (buf[i] != -1)
        buf[i+1] = '\0';
    }
    if (i != 0 && buf[i-1] != -1)
      printf("Invalid.  Just type in the values on the dice that you"
             " want to reroll.\n");
    else
      return;
  }
}

static int get_scoring(int scores[NTITLES]) {
  while (1) {
    char buf[100];
    int i;

    printf("And how do you want to score that? "); fflush(stdout);
    gets(buf);
    for (i=ONES; i<NMOVES; i++)
      if (!strcasecmp(buf, titles[i]) && (scores[i] == -1 || (scores[i] == 50 && i == YAHTZEE)))
        return i;
    printf("Invalid.  Type in a category as it appears above.\n");
  }
}

static int tally(int which, int dice[5]) {
  int i, count = 0;
  for (i=0; i<5; i++)
    if (dice[i] == which)
      count++;
  return count*which;
}

static int of_a_kind(int how_many, int dice[5]) {
  int count[6] = { 0, 0, 0, 0, 0, 0 };
  int i, total = 0;
  for (i=0; i<5; i++)
    total += dice[i];
  for (i=0; i<5; i++)
    if (++count[dice[i]-1] >= how_many)
      return total;
  return 0;
}

static int full_house(int dice[5]) {
  int count[6] = { 0, 0, 0, 0, 0, 0 };
  int i, three=0, two=0;
  for (i=0; i<5; i++)
    count[dice[i]-1]++;
  for (i=0; i<6; i++)
    switch (count[i]) {
      case 5: three = two = 1; break;
      case 3: three = 1; break;
      case 2: two = 1; break;
    }
  return (three && two) ? 25 : 0;
}  

static int sm_straight(int dice[5]) {
  int count[6] = { 0, 0, 0, 0, 0, 0 };
  int i;
  for (i=0; i<5; i++)
    count[dice[i]-1]++;
  return ((count[0] && count[1] && count[2] && count[3]) ||
         (count[1] && count[2] && count[3] && count[4]) ||
         (count[2] && count[3] && count[4] && count[5])) ? 30 : 0;
}

static int lg_straight(int dice[5]) {
  int count[6] = { 0, 0, 0, 0, 0, 0 };
  int i;
  for (i=0; i<5; i++)
    count[dice[i]-1]++;
  return ((count[0] && count[1] && count[2] && count[3] && count[4]) ||
         (count[1] && count[2] && count[3] && count[4] && count[5]) ||
         (count[2] && count[3] && count[4] && count[5] && count[6])) ? 40 : 0;
}

static void do_scoring(int scoring, int scores[NTITLES], int dice[5]) {
  switch (scoring) {
    case ONES: case TWOS: case THREES: case FOURS: case FIVES: case SIXES:
      scores[scoring] = tally(scoring-ONES+1, dice);
      break;
    case THREE_KIND: case FOUR_KIND:
      scores[scoring] = of_a_kind(scoring - THREE_KIND + 3, dice);
      break;
    case YAHTZEE:
      if (of_a_kind(5, dice)) {
        if (scores[scoring] == 50)
          scores[YAHT_BONUS] += 100;
        else
          scores[scoring] = 50;
      }
      else scores[scoring] = 0;
      break;
    case FULL_HOUSE:
      scores[scoring] = full_house(dice);
      break;
    case SM_STRAIGHT:
      scores[scoring] = sm_straight(dice);
      break;
    case LG_STRAIGHT:
      scores[scoring] = lg_straight(dice);
      break;
    case CHANCE:
      scores[scoring] = of_a_kind(1, dice);
  }
}

void do_move(int player, int scores[NTITLES]) {
  int dice[5], mask[5] = { 1, 1, 1, 1, 1 };
  int scoring;

  roll(dice, mask);
  printf("\nPlayer %d, roll 1: %d %d %d %d %d\n",
    player+1, dice[0], dice[1], dice[2], dice[3], dice[4]);
  get_reroll(dice, mask);
  roll(dice, mask);
  printf("Player %d, roll 2: %d %d %d %d %d\n",
    player+1, dice[0], dice[1], dice[2], dice[3], dice[4]);
  get_reroll(dice, mask);
  roll(dice, mask);
  printf("Player %d, roll 3: %d %d %d %d %d\n",
    player+1, dice[0], dice[1], dice[2], dice[3], dice[4]);
  scoring = get_scoring(scores);
  do_scoring(scoring, scores, dice);
}
